@@ -91,19 +91,21 @@ class FirewallCommand(object):
             self.fw.authorizeAll()
         items = [ ]
         _errors = 0
+        _error_codes = [ ]
         for item in option:
             if parse_method is not None:
                 try:
                     item = parse_method(item)
                 except Exception as msg:
+                    code = FirewallError.get_code(str(msg))
                     if len(option) > 1:
                         self.print_warning("Warning: %s" % msg)
-                        _errors += 1
-                        continue
                     else:
-                        code = FirewallError.get_code(str(msg))
                         self.print_and_exit("Error: %s" % msg, code)
-                        _errors += 1
+                    if code not in _error_codes:
+                        _error_codes.append(code)
+                    _errors += 1
+                    continue
 
             items.append(item)
 
@@ -120,38 +122,42 @@ class FirewallCommand(object):
             self.deactivate_exception_handler()
             try:
                 action_method(*call_item)
-            except DBusException as msg:
-                if len(option) > 1:
-                    self.print_warning("Warning: %s" % msg.get_dbus_message())
+            except (DBusException, Exception) as msg:
+                if isinstance(msg, DBusException):
+                    self.fail_if_not_authorized(msg.get_dbus_name())
+                    msg = msg.get_dbus_message()
                 else:
-                    code = FirewallError.get_code(msg.get_dbus_message())
-                    if code in [ errors.ALREADY_ENABLED, errors.NOT_ENABLED,
-                                 errors.ZONE_ALREADY_SET ]:
-                        self.print_warning("Warning: %s" % \
-                                           msg.get_dbus_message())
-                        sys.exit(0)
-                    else:
-                        self.print_and_exit("Error: %s" % \
-                                            msg.get_dbus_message(), code)
-                _errors += 1
-            except Exception as msg:
+                    msg = str(msg)
+                code = FirewallError.get_code(msg)
+                if code in [ errors.ALREADY_ENABLED, errors.NOT_ENABLED,
+                             errors.ZONE_ALREADY_SET ]:
+                    code = 0
                 if len(option) > 1:
                     self.print_warning("Warning: %s" % msg)
-                    _errors += 1
-                    continue
+                elif code == 0:
+                    self.print_warning("Warning: %s" % msg)
+                    return
                 else:
-                    code = FirewallError.get_code(str(msg))
-                    if code in [ errors.ALREADY_ENABLED, errors.NOT_ENABLED,
-                                 errors.ZONE_ALREADY_SET ]:
-                        self.print_warning("Warning: %s" % msg)
-                        sys.exit(0)
-                    else:
-                        self.print_and_exit("Error: %s" % msg, code)
-                    _errors += 1
+                    self.print_and_exit("Error: %s" % msg, code)
+                if code not in _error_codes:
+                    _error_codes.append(code)
+                _errors += 1
             self.activate_exception_handler()
 
-        if _errors == len(option) and not no_exit:
-            sys.exit(errors.UNKNOWN_ERROR)
+        if not no_exit:
+            if len(option) > _errors or 0 in _error_codes:
+                # There have been more options than errors or there
+                # was at least one error code 0, return.
+                return
+            elif len(_error_codes) == 1:
+                # Exactly one error code, use it.
+                sys.exit(_error_codes[0])
+            elif len(_error_codes) > 1:
+                # There is more than error, exit using
+                # UNKNOWN_ERROR. This could happen within sequences
+                # where parsing failed with different errors like
+                # INVALID_PORT and INVALID_PROTOCOL.
+                sys.exit(errors.UNKNOWN_ERROR)
 
     def add_sequence(self, option, action_method, query_method, parse_method,
                      message, no_exit=False):
@@ -211,6 +217,7 @@ class FirewallCommand(object):
             try:
                 res = query_method(*call_item)
             except DBusException as msg:
+                self.fail_if_not_authorized(msg.get_dbus_name())
                 code = FirewallError.get_code(msg.get_dbus_message())
                 if len(option) > 1:
                     self.print_warning("Warning: %s" % msg.get_dbus_message())
@@ -332,6 +339,24 @@ class FirewallCommand(object):
                                 (value, "', '".join(ipvs)))
         return value
 
+    def check_helper_family(self, value):
+        ipvs = [ "", "ipv4", "ipv6" ]
+        if value not in ipvs:
+            raise FirewallError(errors.INVALID_IPV,
+                                "invalid argument: %s (choose from '%s')" % \
+                                (value, "', '".join(ipvs)))
+        return value
+
+    def check_module(self, value):
+        if not value.startswith("nf_conntrack_"):
+            raise FirewallError(
+                errors.INVALID_MODULE,
+                "'%s' does not start with 'nf_conntrack_'" % value)
+        if len(value.replace("nf_conntrack_", "")) < 1:
+            raise FirewallError(errors.INVALID_MODULE,
+                                "Module name '%s' too short" % value)
+        return value
+
     def print_zone_info(self, zone, settings, default_zone=None):
         target = settings.getTarget()
         icmp_block_inversion = settings.getIcmpBlockInversion()
@@ -375,7 +400,7 @@ class FirewallCommand(object):
                                     (port, proto, toport, toaddr)
                                     for (port, proto, toport, toaddr) in \
                                     forward_ports]))
-        self.print_msg("  sourceports: " +
+        self.print_msg("  source-ports: " +
                        " ".join(["%s/%s" % (port[0], port[1])
                                  for port in source_ports]))
         self.print_msg("  icmp-blocks: " + " ".join(icmp_blocks))
@@ -431,6 +456,21 @@ class FirewallCommand(object):
                                                  for k, v in options.items()]))
         self.print_msg("  entries: " + " ".join(entries))
 
+    def print_helper_info(self, helper, settings):
+        ports = settings.getPorts()
+        module = settings.getModule()
+        family = settings.getFamily()
+        description = settings.getDescription()
+        short_description = settings.getShort()
+        self.print_msg(helper)
+        if self.verbose:
+            self.print_msg("  summary: " + short_description)
+            self.print_msg("  description: " + description)
+        self.print_msg("  family: " + family)
+        self.print_msg("  module: " + module)
+        self.print_msg("  ports: " + " ".join(["%s/%s" % (port[0], port[1])
+                                               for port in ports]))
+
     def print_query_result(self, value):
         if value:
             self.print_and_exit("yes")
@@ -440,17 +480,19 @@ class FirewallCommand(object):
     def exception_handler(self, exception_message):
         if not self.__use_exception_handler:
             raise
+        self.fail_if_not_authorized(exception_message)
+        code = FirewallError.get_code(str(exception_message))
+        if code in [ errors.ALREADY_ENABLED, errors.NOT_ENABLED,
+                     errors.ZONE_ALREADY_SET ]:
+            self.print_warning("Warning: %s" % exception_message)
+        else:
+            self.print_and_exit("Error: %s" % exception_message, code)
+
+    def fail_if_not_authorized(self, exception_message):
         if "NotAuthorizedException" in exception_message:
             msg = """Authorization failed.
     Make sure polkit agent is running or run the application as superuser."""
             self.print_and_exit(msg, errors.NOT_AUTHORIZED)
-        else:
-            code = FirewallError.get_code(str(exception_message))
-            if code in [ errors.ALREADY_ENABLED, errors.NOT_ENABLED,
-                         errors.ZONE_ALREADY_SET ]:
-                self.print_warning("Warning: %s" % exception_message)
-            else:
-                self.print_and_exit("Error: %s" % exception_message, code)
 
     def deactivate_exception_handler(self):
         self.__use_exception_handler = False
