@@ -38,6 +38,7 @@ from firewall.core.fw_config import FirewallConfig
 from firewall.core.fw_policies import FirewallPolicies
 from firewall.core.fw_ipset import FirewallIPSet
 from firewall.core.fw_transaction import FirewallTransaction, reverse_rule
+from firewall.core.fw_helper import FirewallHelper
 from firewall.core.logger import log
 from firewall.core.io.firewalld_conf import firewalld_conf
 from firewall.core.io.direct import Direct
@@ -45,6 +46,7 @@ from firewall.core.io.service import service_reader
 from firewall.core.io.icmptype import icmptype_reader
 from firewall.core.io.zone import zone_reader, Zone
 from firewall.core.io.ipset import ipset_reader
+from firewall.core.io.helper import helper_reader
 from firewall import errors
 from firewall.errors import FirewallError
 
@@ -82,6 +84,7 @@ class Firewall(object):
         self.config = FirewallConfig(self)
         self.policies = FirewallPolicies()
         self.ipset = FirewallIPSet(self)
+        self.helper = FirewallHelper(self)
 
         self.__init_vars()
 
@@ -91,7 +94,8 @@ class Firewall(object):
              self.ebtables_enabled, self._state, self._panic,
              self._default_zone, self._module_refcount, self._marks,
              self._min_mark, self.cleanup_on_exit, self.ipv6_rpfilter_enabled,
-             self.ipset_enabled, self._individual_calls, self._log_denied)
+             self.ipset_enabled, self._individual_calls, self._log_denied,
+             self._automatic_helpers)
 
     def __init_vars(self):
         self._state = "INIT"
@@ -105,6 +109,9 @@ class Firewall(object):
         self.ipv6_rpfilter_enabled = config.FALLBACK_IPV6_RPFILTER
         self._individual_calls = config.FALLBACK_INDIVIDUAL_CALLS
         self._log_denied = config.FALLBACK_LOG_DENIED
+        self._automatic_helpers = config.FALLBACK_AUTOMATIC_HELPERS
+        self.nf_conntrack_helper_setting = 0
+        self.nf_conntrack_helpers = { }
 
     def individual_calls(self):
         return self._individual_calls
@@ -123,7 +130,7 @@ class Firewall(object):
 
         if self.ebtables_enabled and \
            "filter" not in self.get_available_tables("eb"):
-            log.error("ebtables not usable, disabling ethernet bridge firewall.")
+            log.warning("ebtables not usable, disabling ethernet bridge firewall.")
             self.ebtables_enabled = False
 
         # is there at least support for ipv4 or ipv6
@@ -135,7 +142,7 @@ class Firewall(object):
         try:
             self.ipset_backend.list()
         except ValueError:
-            log.error("ipset not usable, disabling ipset usage in firewall.")
+            log.warning("ipset not usable, disabling ipset usage in firewall.")
             # ipset is not usable, no supported types
             self.ipset_enabled = False
             self.ipset_supported_types = [ ]
@@ -143,6 +150,53 @@ class Firewall(object):
             # ipset is usable, get all supported types
             self.ipset_supported_types = self.ipset_backend.supported_types()
 
+        self.ip4tables_backend.fill_exists()
+        if not self.ip4tables_backend.restore_command_exists:
+            if self.ip4tables_backend.command_exists:
+                log.warning("iptables-restore is missing, using "
+                            "individual calls for IPv4 firewall.")
+            else:
+                log.warning("iptables-restore and iptables are missing, "
+                            "disabling IPv4 firewall.")
+                self.ip4tables_enabled = False
+
+        self.ip6tables_backend.fill_exists()
+        if not self.ip6tables_backend.restore_command_exists:
+            if self.ip6tables_backend.command_exists:
+                log.warning("ip6tables-restore is missing, using "
+                            "individual calls for IPv6 firewall.")
+            else:
+                log.warning("ip6tables-restore and ip6tables are missing, "
+                            "disabling IPv6 firewall.")
+                self.ip6tables_enabled = False
+
+        self.ebtables_backend.fill_exists()
+        if not self.ebtables_backend.restore_command_exists:
+            if self.ebtables_backend.command_exists:
+                log.warning("ebtables-restore is missing, using "
+                            "individual calls for bridge firewall.")
+            else:
+                log.warning("ebtables-restore and ebtables are missing, "
+                            "disabling bridge firewall.")
+                self.ebtables_enabled = False
+
+        if self.ebtables_enabled and not self._individual_calls and \
+           not self.ebtables_backend.restore_noflush_option:
+            log.debug1("ebtables-restore is not supporting the --noflush "
+                       "option, will therefore not be used")
+
+        if os.path.exists(config.COMMANDS["modinfo"]):
+            self.nf_conntrack_helpers = functions.get_nf_conntrack_helpers()
+            if len(self.nf_conntrack_helpers) > 0:
+                log.debug1("Conntrack helpers supported by the kernel:")
+                for key,values in self.nf_conntrack_helpers.items():
+                    log.debug1("  %s: %s", key, ", ".join(values))
+            else:
+                log.debug1("No conntrack helpers supported by the kernel.")
+        else:
+            self.nf_conntrack_helpers = { }
+            log.warning("modinfo command is missing, not able to detect conntrack helpers.")
+
     def _start(self, reload=False, complete_reload=False):
         # initialize firewall
         default_zone = config.FALLBACK_ZONE
@@ -152,6 +206,7 @@ class Firewall(object):
         try:
             self._firewalld_conf.read()
         except Exception as msg:
+            log.warning(msg)
             log.warning("Using fallback firewalld configuration settings.")
         else:
             if self._firewalld_conf.get("DefaultZone"):
@@ -201,9 +256,17 @@ class Firewall(object):
                     self._log_denied = value.lower()
                     log.debug1("LogDenied is set to '%s'", self._log_denied)
 
-            if self.ebtables_enabled and not self._individual_calls and \
-               not self.ebtables_backend.restore_noflush_option:
-                log.debug1("ebtables-restore is not supporting the --noflush option, will therefore not be used")
+            if self._firewalld_conf.get("AutomaticHelpers"):
+                value = self._firewalld_conf.get("AutomaticHelpers")
+                if value is None:
+                    if value.lower() in [ "no", "false" ]:
+                        self._automatic_helpers = "no"
+                    if value.lower() in [ "yes", "true" ]:
+                        self._automatic_helpers = "yes"
+                else:
+                    self._automatic_helpers = value.lower()
+                log.debug1("AutomaticHelpers is set to '%s'",
+                           self._automatic_helpers)
 
         self.config.set_firewalld_conf(copy.deepcopy(self._firewalld_conf))
 
@@ -235,6 +298,10 @@ class Firewall(object):
         if len(self.icmptype.get_icmptypes()) == 0:
             log.error("No icmptypes found.")
 
+        # load helper files
+        self._loader(config.FIREWALLD_HELPERS, "helper")
+        self._loader(config.ETC_FIREWALLD_HELPERS, "helper")
+
         # load service files
         self._loader(config.FIREWALLD_SERVICES, "service")
         self._loader(config.ETC_FIREWALLD_SERVICES, "service")
@@ -287,6 +354,12 @@ class Firewall(object):
         self.direct.set_permanent_config(obj)
         self.config.set_direct(copy.deepcopy(obj))
 
+        # automatic helpers
+        if self._automatic_helpers != "system":
+            functions.set_nf_conntrack_helper_setting(self._automatic_helpers == "yes")
+        self.nf_conntrack_helper_setting = \
+            functions.get_nf_conntrack_helper_setting()
+
         # check if needed tables are there
         self._check_tables()
 
@@ -468,6 +541,19 @@ class Firewall(object):
                     self.ipset.add_ipset(obj)
                     # add a deep copy to the configuration interface
                     self.config.add_ipset(copy.deepcopy(obj))
+                elif reader_type == "helper":
+                    obj = helper_reader(filename, path)
+                    if obj.name in self.helper.get_helpers():
+                        orig_obj = self.helper.get_helper(obj.name)
+                        log.debug1("  Overloads %s '%s' ('%s/%s')", reader_type,
+                                   orig_obj.name, orig_obj.path,
+                                   orig_obj.filename)
+                        self.helper.remove_helper(orig_obj.name)
+                    elif obj.path.startswith(config.ETC_FIREWALLD):
+                        obj.default = True
+                    self.helper.add_helper(obj)
+                    # add a deep copy to the configuration interface
+                    self.config.add_helper(copy.deepcopy(obj))
                 else:
                     log.fatal("Unknown reader type %s", reader_type)
             except FirewallError as msg:
@@ -500,6 +586,7 @@ class Firewall(object):
         self.service.cleanup()
         self.zone.cleanup()
         self.ipset.cleanup()
+        self.helper.cleanup()
         self.config.cleanup()
         self.direct.cleanup()
         self.policies.cleanup()
@@ -596,6 +683,12 @@ class Firewall(object):
 
         if self.ipv6_rpfilter_enabled and \
            "raw" in self.get_available_tables("ipv6"):
+
+            # Execute existing transaction
+            transaction.execute(True)
+            # Start new transaction
+            transaction.clear()
+
             # here is no check for ebtables.restore_noflush_option needed
             # as ebtables is not used in here
             transaction.add_rule("ipv6",
@@ -613,8 +706,17 @@ class Firewall(object):
                                        "-j", "LOG",
                                        "--log-prefix", "rpfilter_DROP: " ])
 
-        if use_transaction is None:
-            transaction.execute(True)
+            # Execute ipv6_rpfilter transaction, it might fail
+            try:
+                transaction.execute(True)
+            except FirewallError as msg:
+                log.warning("Applying rules for ipv6_rpfilter failed: %s", msg)
+            # Start new transaction
+            transaction.clear()
+
+        else:
+            if use_transaction is None:
+                transaction.execute(True)
 
     # flush and policy
 
@@ -814,6 +916,7 @@ class Firewall(object):
             return ""
 
         if self._individual_calls or \
+           not backend.restore_command_exists or \
            (ipv == "eb" and not self.ebtables_backend.restore_noflush_option):
             for i,rule in enumerate(_rules):
                 # remove leading and trailing '"' for use with execve
@@ -1014,6 +1117,27 @@ class Firewall(object):
         else:
             raise FirewallError(errors.ALREADY_SET, value)
 
+    # AUTOMATIC HELPERS
+
+    def get_automatic_helpers(self):
+        return self._automatic_helpers
+
+    def set_automatic_helpers(self, value):
+        if value not in config.AUTOMATIC_HELPERS_VALUES:
+            raise FirewallError(errors.INVALID_VALUE,
+                                "'%s', choose from '%s'" % \
+                                (value, "','".join(config.AUTOMATIC_HELPERS_VALUES)))
+
+        if value != self.get_automatic_helpers():
+            self._automatic_helpers = value
+            self._firewalld_conf.set("AutomaticHelpers", value)
+            self._firewalld_conf.write()
+
+            # now reload the firewall
+            self.reload()
+        else:
+            raise FirewallError(errors.ALREADY_SET, value)
+
     # DEFAULT ZONE
 
     def get_default_zone(self):
