@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2011-2012 Red Hat, Inc.
+# Copyright (C) 2011-2016 Red Hat, Inc.
 #
 # Authors:
 # Thomas Woerner <twoerner@redhat.com>
@@ -19,15 +19,21 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+"""Generic io_object handler, io specific check methods."""
+
+__all__ = [ "PY2",
+            "IO_Object", "IO_Object_ContentHandler", "IO_Object_XMLGenerator",
+            "check_port", "check_tcpudp", "check_protocol", "check_address" ]
+
 import xml.sax as sax
 import xml.sax.saxutils as saxutils
 import copy
 import sys
 
-from firewall.config import *
-from firewall.errors import *
 from firewall import functions
 from firewall.functions import b2u
+from firewall import errors
+from firewall.errors import FirewallError
 
 PY2 = sys.version < '3'
 
@@ -44,6 +50,8 @@ class IO_Object(object):
         self.filename = ""
         self.path = ""
         self.name = ""
+        self.default = False
+        self.builtin = False
 
     def export_config(self):
         ret = [ ]
@@ -51,68 +59,75 @@ class IO_Object(object):
             ret.append(copy.deepcopy(getattr(self, x[0])))
         return tuple(ret)
 
-    def import_config(self, config):
-        self.check_config(config)
-        for i,(element,value) in enumerate(self.IMPORT_EXPORT_STRUCTURE):
-            if isinstance(config[i], list):
-                # remove duplicates
-                setattr(self, element, copy.deepcopy(list(set(config[i]))))
+    def import_config(self, conf):
+        self.check_config(conf)
+        for i,(element,dummy) in enumerate(self.IMPORT_EXPORT_STRUCTURE):
+            if isinstance(conf[i], list):
+                # remove duplicates without changing the order
+                _conf = [ ]
+                for x in conf[i]:
+                    if x not in _conf:
+                        _conf.append(x)
+                setattr(self, element, copy.deepcopy(_conf))
             else:
-                setattr(self, element, copy.deepcopy(config[i]))
+                setattr(self, element, copy.deepcopy(conf[i]))
 
     def check_name(self, name):
         if type(name) != type(""):
-            raise FirewallError(INVALID_TYPE,
+            raise FirewallError(errors.INVALID_TYPE,
                                 "'%s' not of type %s, but %s" % (name, type(""),
                                                                  type(name)))
         if len(name) < 1:
-            raise FirewallError(INVALID_NAME, "name can't be empty")
+            raise FirewallError(errors.INVALID_NAME, "name can't be empty")
         for char in name:
             if not char.isalnum() and char not in self.ADDITIONAL_ALNUM_CHARS:
-                raise FirewallError(INVALID_NAME,
-                                 "'%s' is not allowed in '%s'" % ((char, name)))
+                raise FirewallError(
+                    errors.INVALID_NAME,
+                    "'%s' is not allowed in '%s'" % ((char, name)))
 
-    def check_config(self, config):
-        if len(config) != len(self.IMPORT_EXPORT_STRUCTURE):
-            raise FirewallError(INVALID_TYPE,
-                                "structure size mismatch %d != %d" % (\
-                    len(config), len(self.IMPORT_EXPORT_STRUCTURE)))
+    def check_config(self, conf):
+        if len(conf) != len(self.IMPORT_EXPORT_STRUCTURE):
+            raise FirewallError(
+                errors.INVALID_TYPE,
+                "structure size mismatch %d != %d" % \
+                (len(conf), len(self.IMPORT_EXPORT_STRUCTURE)))
         for i,(element,value) in enumerate(self.IMPORT_EXPORT_STRUCTURE):
-            self._check_config_structure(config[i], value)
-            self._check_config(config[i], element)
+            self._check_config_structure(conf[i], value)
+            self._check_config(conf[i], element)
 
-    def _check_config(self, config, item):
+    def _check_config(self, dummy1, dummy2):
         # to be overloaded by sub classes
         return
 
-    def _check_config_structure(self, config, structure):
-        if not type(config) == type(structure):
-            raise FirewallError(INVALID_TYPE,
-                                "'%s' not of type %s, but %s" % (\
-                    config, type(structure), type(config)))
-        if type(structure) == list:
+    def _check_config_structure(self, conf, structure):
+        if not type(conf) == type(structure):
+            raise FirewallError(errors.INVALID_TYPE,
+                                "'%s' not of type %s, but %s" % \
+                                (conf, type(structure), type(conf)))
+        if isinstance(structure, list):
             # same type elements, else struct
             if len(structure) != 1:
-                raise FirewallError(INVALID_TYPE, "len('%s') != 1" % structure)
-            for x in config:
+                raise FirewallError(errors.INVALID_TYPE,
+                                    "len('%s') != 1" % structure)
+            for x in conf:
                 self._check_config_structure(x, structure[0])
-        elif type(structure) == tuple:
-            if len(structure) != len(config):
-                raise FirewallError(INVALID_TYPE,
-                                    "len('%s') != %d" % (config,
+        elif isinstance(structure, tuple):
+            if len(structure) != len(conf):
+                raise FirewallError(errors.INVALID_TYPE,
+                                    "len('%s') != %d" % (conf,
                                                          len(structure)))
             for i,value in enumerate(structure):
-                self._check_config_structure(config[i], value)
-        elif type(structure) == dict:
+                self._check_config_structure(conf[i], value)
+        elif isinstance(structure, dict):
             # only one key value pair in structure
             (skey, svalue) = list(structure.items())[0]
-            for (key, value) in config.items():
+            for (key, value) in conf.items():
                 if type(key) != type(skey):
-                    raise FirewallError(INVALID_TYPE,
+                    raise FirewallError(errors.INVALID_TYPE,
                                         "'%s' not of type %s, but %s" % (\
                             key, type(skey), type(key)))
                 if type(value) != type(svalue):
-                    raise FirewallError(INVALID_TYPE,
+                    raise FirewallError(errors.INVALID_TYPE,
                                         "'%s' not of type %s, but %s" % (\
                             value, type(svalue), type(value)))
 
@@ -129,52 +144,57 @@ class IO_Object(object):
                         _attrs.remove(x)
                     else:
                         raise FirewallError(
-                            PARSE_ERROR, "Missing attribute %s for %s" % 
-                            (x, name))
+                            errors.PARSE_ERROR,
+                            "Missing attribute %s for %s" % (x, name))
         if name in self.PARSER_OPTIONAL_ELEMENT_ATTRS:
             found = True
             for x in self.PARSER_OPTIONAL_ELEMENT_ATTRS[name]:
                 if x in _attrs:
                     _attrs.remove(x)
         if not found:
-            raise FirewallError(PARSE_ERROR, "Unexpected element %s" % name)
+            raise FirewallError(errors.PARSE_ERROR,
+                                "Unexpected element %s" % name)
         # raise attributes[0]
         for x in _attrs:
-            raise FirewallError(PARSE_ERROR, "%s: Unexpected attribute %s" % 
-                                (name, x))
- 
+            raise FirewallError(errors.PARSE_ERROR,
+                                "%s: Unexpected attribute %s" % (name, x))
+
 # PARSER
 
 class UnexpectedElementError(Exception):
     def __init__(self, name):
+        super(UnexpectedElementError, self).__init__()
         self.name = name
     def __str__(self):
-        return _("Unexpected element '%s'") % (self.name)
+        return "Unexpected element '%s'" % (self.name)
 
 class MissingAttributeError(Exception):
     def __init__(self, name, attribute):
+        super(MissingAttributeError, self).__init__()
         self.name = name
         self.attribute = attribute
     def __str__(self):
-        return _("Element '%s': missing '%s' attribute") % \
+        return "Element '%s': missing '%s' attribute" % \
             (self.name, self.attribute)
 
 class UnexpectedAttributeError(Exception):
     def __init__(self, name, attribute):
+        super(UnexpectedAttributeError, self).__init__()
         self.name = name
         self.attribute = attribute
     def __str__(self):
-        return _("Element '%s': unexpected attribute '%s'") % \
+        return "Element '%s': unexpected attribute '%s'" % \
             (self.name, self.attribute)
 
 class IO_Object_ContentHandler(sax.handler.ContentHandler):
     def __init__(self, item):
         self.item = item
+        self._element = ""
 
     def startDocument(self):
         self._element = ""
 
-    def startElement(self, name):
+    def startElement(self, name, attrs):
         self._element = ""
 
     def endElement(self, name):
@@ -221,7 +241,7 @@ class IO_Object_XMLGenerator(saxutils.XMLGenerator):
             self._write(u'<' + b2u(name))
             for (name, value) in attrs.items():
                 self._write(u' %s=%s' % (b2u(name),
-                                        saxutils.quoteattr(b2u(value))))
+                                         saxutils.quoteattr(b2u(value))))
             self._write(u'/>')
         else:
             self._write('<' + name)
@@ -256,27 +276,29 @@ class IO_Object_XMLGenerator(saxutils.XMLGenerator):
 def check_port(port):
     port_range = functions.getPortRange(port)
     if port_range == -2:
-        raise FirewallError(INVALID_PORT,
+        raise FirewallError(errors.INVALID_PORT,
                             "port number in '%s' is too big" % port)
     elif port_range == -1:
-        raise FirewallError(INVALID_PORT, "'%s' is invalid port range" % port)
+        raise FirewallError(errors.INVALID_PORT,
+                            "'%s' is invalid port range" % port)
     elif port_range is None:
-        raise FirewallError(INVALID_PORT,
+        raise FirewallError(errors.INVALID_PORT,
                             "port range '%s' is ambiguous" % port)
     elif len(port_range) == 2 and port_range[0] >= port_range[1]:
-        raise FirewallError(INVALID_PORT, "'%s' is invalid port range" % port)
+        raise FirewallError(errors.INVALID_PORT,
+                            "'%s' is invalid port range" % port)
 
 def check_tcpudp(protocol):
     if protocol not in [ "tcp", "udp" ]:
-        raise FirewallError(INVALID_PROTOCOL,
+        raise FirewallError(errors.INVALID_PROTOCOL,
                             "'%s' not from {'tcp'|'udp'}" % protocol)
 
 def check_protocol(protocol):
     if not functions.checkProtocol(protocol):
-        raise FirewallError(INVALID_PROTOCOL, protocol)
+        raise FirewallError(errors.INVALID_PROTOCOL, protocol)
 
 def check_address(ipv, addr):
     if not functions.check_address(ipv, addr):
-        raise FirewallError(INVALID_ADDR,
+        raise FirewallError(errors.INVALID_ADDR,
                             "'%s' is not valid %s address" % (addr, ipv))
 
