@@ -156,6 +156,20 @@ assert_rich_bad() {
   fi
 }
 
+assert_exit_code() {
+  local args="${1}"
+  local ret="${2}"
+
+  ${path}firewall-cmd ${args} > /dev/null 2>&1
+  got=$?
+  if [[ "$got" -eq "$ret" ]]; then
+    echo "${args} ... OK"
+  else
+    ((failures++))
+    echo -e "${args} ... ${RED}${failures}. FAILED (bad exit status ${got} != ${ret})${RESTORE}"
+  fi
+}
+
 test_lokkit_opts() {
 rm -f /etc/firewalld/zones/*
 assert_good "${lokkit_opts}"
@@ -455,13 +469,12 @@ assert_good "--service=${myservice} --query-source-port=111-222/udp"
 assert_good "--service=${myservice} --remove-source-port 111-222/udp"
 assert_bad  "--service=${myservice} --query-source-port=111-222/udp"
 
-assert_bad  "--service=${myservice} --add-module=foo" # no valid helper
-assert_good "--service=${myservice} --add-module=nf_conntrack_sip"
-assert_good "--service=${myservice} --remove-module=nf_conntrack_sip"
-assert_good "--service=${myservice} --add-module=nf_conntrack_ftp"
-assert_good "--service=${myservice} --query-module=nf_conntrack_ftp"
-assert_good "--service=${myservice} --remove-module=nf_conntrack_ftp"
-assert_bad "--service=${myservice} --query-module=nf_conntrack_ftp"
+assert_good "--service=${myservice} --add-module=sip"
+assert_good "--service=${myservice} --remove-module=sip"
+assert_good "--service=${myservice} --add-module=ftp"
+assert_good "--service=${myservice} --query-module=ftp"
+assert_good "--service=${myservice} --remove-module=ftp"
+assert_bad "--service=${myservice} --query-module=ftp"
 
 assert_bad  "--service=${myservice} --set-destination=ipv4" # no address
 assert_bad  "--service=${myservice} --set-destination=ipv4:foo" # bad address
@@ -515,6 +528,41 @@ assert_good "--zone=${zone} --remove-source=${source}"
 
 assert_good "--delete-ipset=${ipset}"
 
+# helper tests
+myhelper="myhelper"
+assert_bad "--new-helper=${myhelper} --module=foo"
+assert_good "--new-helper=${myhelper} --module=nf_conntrack_foo"
+assert_good_contains "--get-helpers" "${myhelper}"
+assert_good_empty "--helper=${myhelper} --get-family"
+assert_bad "--helper=${myhelper} --set-family=ipv5"
+assert_good "--helper=${myhelper} --set-family=ipv4"
+assert_good_equals "--helper=${myhelper} --get-family" "ipv4"
+assert_good "--helper=${myhelper} --set-family="
+assert_good_empty "--helper=${myhelper} --get-family"
+assert_good_empty "--helper=${myhelper} --get-ports"
+assert_good "--helper=${myhelper} --add-port=44/tcp"
+assert_good_notempty "--helper=${myhelper} --get-ports"
+assert_good "--helper=${myhelper} --query-port=44/tcp"
+assert_good "--helper=${myhelper} --remove-port=44/tcp"
+assert_bad "--helper=${myhelper} --query-port=44/tcp"
+assert_good_empty "--helper=${myhelper} --get-ports"
+assert_good "--delete-helper=${myhelper}"
+assert_bad_contains "--get-helpers" "${myhelper}"
+
+# exit return value tests
+assert_exit_code "--remove-port 122/udp" 0
+assert_exit_code "--add-port 122/udpp" 103
+assert_exit_code "--add-port 122/udp --add-port 122/udpp" 0
+assert_exit_code "--add-port 122/udp --add-port 122/udpp" 0
+assert_exit_code "--add-port 122/udp --add-port 122/udpp --add-port 8745897/foo" 0
+assert_exit_code "--add-port 122/udp --add-port 122/udpp --add-port 8745897/foo --add-port bar" 0
+assert_exit_code "--add-port 122/udpa --add-port 122/udpp" 103
+assert_exit_code "--add-port 122/udpa --add-port 122/udpp" 103
+assert_exit_code "--add-port 122/udpa --add-port 122/udpp --add-port 8745897/foo" 254
+assert_exit_code "--add-port 122/udpa --add-port 122/udpp --add-port 8745897/foo --add-port bar" 254
+assert_exit_code "--add-port 122/udp --add-port 122/udp" 0
+assert_exit_code "--remove-port 122/udp" 0
+
 # ... --direct ...
 assert_bad           "--direct --add-passthrough ipv7 --table filter -A INPUT --in-interface dummy0 --protocol tcp --destination-port 67 --jump ACCEPT" # bad ipv
 assert_good          "--direct --add-passthrough ipv4 --table filter --append INPUT --in-interface dummy0 --protocol tcp --destination-port 67 --jump ACCEPT"
